#!/usr/bin/perl

use Data::Dumper;
use CGI;
use Digest::SHA1 qw(sha1_hex);
use MIME::Base64;
use XML::DOM;

sub sha1hex_to_base64 {
  my $in = shift;
  my $tmp = '';
  while ($in =~ s/^(..)//) {
    $tmp .= chr(hex($1));
  }
  return encode_base64($tmp, '');
}

sub writehtpasswd {
  my $user = shift;
  my $passwordhash = shift;
  if (($passwordhash eq "") && ($user eq "")) {
  } elsif (($passwordhash eq "") && ($user ne "")) {
    $passwordhash = "{SHA}" . sha1hex_to_base64(sha1_hex(""));
  } else {
    $passwordhash = "{SHA}" . sha1hex_to_base64($passwordhash);
  }

  my $fname = ".htpasswd";
  open CONFFILE, ">" . $fname || die("Nem tudom megnyitni a jelszfile-t! ($fname)");
  print CONFFILE $user . ":" . $passwordhash . "\n" || die("Hiba a jelszfile kirsnl ($fname)");
  close CONFFILE || die("Nem tudom lezrni a jelszfile-t! ($fname)");
}

sub config_checklogin {
  my $passwordarray = shift;
  my $user = (keys %{$passwordarray})[0];
  my $passwordhash = %{$passwordarray}->{$user};
  if (defined($ENV{'REMOTE_USER'})) {
    if ($ENV{'REMOTE_USER'} ne $user) {
      writehtpasswd($user, $passwordhash);
      print "ismeretlen hiba a felhasznl azonostsakor\n<br />\n";
      print "krem prblja meg frissteni a bngszablakot (refresh)\n<br />\n";
      exit;
    }
  } else {
    writehtpasswd($user, $passwordhash);
    print "kritikus hiba a .htaccess vagy a .htpasswd file-ban\n";
    exit;
  }
}

sub getSubelementContent {
  my ($DomElement, $tagname, $num) = @_;
  if (defined($DomElement)) {
    @x = $DomElement->getElementsByTagName($tagname);
    if ($#x >= $num) {
      if ($x[$num]->getFirstChild()) {
        return $x[$num]->getFirstChild()->toString();
      } else {
        return "";
      }
    }
  }
  return undef;
}

sub h2a {
  my $tmp = shift;
  my @out;
  foreach $val (keys %{$tmp}) {
    push @out, $val;
  }
  return join(",", @out);
}

sub getConnectIPs {
  my ($dom, $tagname) = @_;

  my $allow;
  my @domallow = $dom->getElementsByTagName($tagname);
  if ($domallow[0]) {
    my @domHCip = ($dom->getElementsByTagName($tagname))[0]->getChildNodes;
    for my $kid (@domHCip) {
      if (($kid->getNodeType == ELEMENT_NODE) && ($kid->getTagName() eq "ip")) {
        $allow->{$kid->getFirstChild()->toString()} = $kid->getFirstChild()->toString();
      }
    }
  }
  return $allow;
}

sub loadConfig {
  my $XMLconfigfile = shift;

  my $parser = new XML::DOM::Parser;
  if (! -e $XMLconfigfile) {
    return undef
  }
  my $dom = $parser->parsefile($XMLconfigfile) or return undef;
  #die "Hiba a konfiguracio betoltese kozben( " . $XMLconfigfile . " )\n";
  my $cfg;

  #$cfg = Array();

  # Database
  my @domdatabase = $dom->getElementsByTagName("database");
  $cfg->{"db.type"} = getSubelementContent($domdatabase[0], "type");
  $cfg->{"db.host"} = getSubelementContent($domdatabase[0], "host");
  $cfg->{"db.username"} = getSubelementContent($domdatabase[0], "username");
  $cfg->{"db.password"} = getSubelementContent($domdatabase[0], "password");
  $cfg->{"db.database"} = getSubelementContent($domdatabase[0], "database");
  $cfg->{"db.table"} = getSubelementContent($domdatabase[0], "table");
  if (getSubelementContent($domdatabase[0], "warn") eq "1") {
    $cfg->{"db.warn"} = 1;
  } else {
    $cfg->{"db.warn"} = 0;
  }

  # Settings / Identification
  my @domsettings = $dom->getElementsByTagName("settings");
  $cfg->{"sett.res.size"} = getSubelementContent($domsettings[0], "resumptionSize");
  $cfg->{"sett.res.exp"} = getSubelementContent($domsettings[0], "resumptionExpirationHours");

  my @domidentification = $dom->getElementsByTagName("identification");
  $cfg->{"id.repositoryName"} = getSubelementContent($domidentification[0], "repositoryName");
  $cfg->{"id.repositoryId"} = getSubelementContent($domidentification[0], "repositoryId");
  $cfg->{"id.baseURL"} = getSubelementContent($domidentification[0], "baseURL");
  $cfg->{"id.protocolVersion"} = getSubelementContent($domidentification[0], "protocolVersion");
  $cfg->{"id.del"} = getSubelementContent($domidentification[0], "deletedRecord");
  $cfg->{"id.email"} = getSubelementContent($domidentification[0], "adminEmail");
  $i = 0;
  while ($tmp = getSubelementContent($domidentification[0], "friends", $i)) {
    push @{$cfg->{"id.friends"}}, $tmp;
    $i++;
  }

  # Manager hozzfrs
  my @dommanager = ($dom->getElementsByTagName("manager"));
  $cfg->{"man.user"} = getSubelementContent($dommanager[0], "user");
  $cfg->{"man.passwordHash"} = getSubelementContent($dommanager[0], "passwordHash");

  # Vltozsjelents
  my @dommodify = $dom->getElementsByTagName("modificationReport");
  if($#dommodify >= 0) {
    $cfg->{"mod.url"} = getSubelementContent($dommodify[0], "url");
    $cfg->{"mod.idParParam"} = getSubelementContent($dommodify[0], "identifierParamName");
    $cfg->{"mod.schemaParName"} = getSubelementContent($dommodify[0], "schemaListParamName");
  }

  # Metaadat formtumok.
  @dommetadataFormats = $dom->getElementsByTagName("metadataFormats");
  if ($#dommetadataFormats >= 0) {
    @domFormats = $dommetadataFormats[0]->getChildNodes();
    my $formatnum = 0;
    for my $value (@domFormats) {
      if (($value->getNodeType == ELEMENT_NODE) && ($value->getTagName() eq "format")) {
        my $format;

        $format->{"name"} = $value->getAttribute("name");
        $format->{"rootElementName"} = getSubelementContent($value, "rootElementName");
        $format->{"schema"} = getSubelementContent($value, "schema");
        $format->{"namespace"} = getSubelementContent($value, "namespace");
        $format->{"csvseparator"} = getSubelementContent($value, "csvseparator");

        my @domcsvmap = $value->getChildNodes();
        my $csv;
        for my $csvmap (@domcsvmap) {
          if (($csvmap->getNodeType == ELEMENT_NODE) && ($csvmap->getTagName() eq "csvmap")) {
            $i = $csvmap->getAttribute("column");
            $csv->{$i}->{"tag"} = $csvmap->getAttribute("tag");
            $csv->{$i}->{"attribute"} = $csvmap->getAttribute("attribute");
          }
        }
        $format->{"csv"} = $csv;

        #push @{$cfg->{"format"}}, $format;
        #push @{$cfg->{"format"}}, $format;
        $cfg->{"format"}->{$formatnum} = $format;
        $formatnum++;
      }
    }
  }

  $cfg->{"ip.harvester"} = join(",", h2a(getConnectIPs($dom, "allowHarvesterConnect")));
  $cfg->{"ip.manager"} = join(",", h2a(getConnectIPs($dom, "allowManagerConnect")));

  return $cfg;
}

sub mergeArrays {
  my ($a, $b) = @_;
  foreach $key (keys %$b) {
    $val = $b->{$key};
    my $isbhref = 0;
    foreach $x (keys %{$b->{$key}}) {
      $isbhref++;
    }
    my $isahref = 0;
    foreach $x (keys %{$a->{$key}}) {
      $isahref++;
    }
    if($isbhref > 0) {
      if ($isahref > 0) {
        $a->{$key} = mergeArrays($a->{$key}, $b->{$key});
      } else {
        $a->{$key} = $b->{$key};
      }
    } elsif(defined($val)) {
      $a->{$key} = $val;
    }
  }
  return $a;
}

return 1;
