#!/usr/bin/perl

require("parameters_inc.pl");
require("oaicommon.pl");
require("oaidb.pl");
use Data::Dumper;

sub putGeneralHeader {
  print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
  print XMLtagStart('OAI-PMH', {'xmlns', 'http://www.openarchives.org/OAI/2.0/', 'xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance', 'xsi:schemaLocation' ,'http://www.openarchives.org/OAI/2.0/ http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd'}) . "\n";
  print XMLtag('responseDate', undef, ISO8601Date()) . "\n";
  print '<request ';
  @values = split(/&/, $ENV{'QUERY_STRING'});
  foreach $i (@values) {
    ($varname, $mydata) = split(/=/, $i);
    print $varname . "=\"" . $mydata . "\" ";
  }
  print '>' . ScriptURL() . "</request>\n";
}

sub OAIXMLError {
  my ($errorcode, $additional) = @_;
  putGeneralHeader();
  print XMLtag("error", {"code", $errorcode}, $additional);
  print "</OAI-PMH>";
}

sub OAIidentify {
  {
    putGeneralHeader();
    print "<Identify><repositoryName>" . $config->{"repositoryName"} . "</repositoryName>";
    print "<baseURL>" . ScriptURL() . "</baseURL>";
    print "<protocolVersion>2.0</protocolVersion>";
    print "<adminEmail>" . $config->{"adminEmail"} . "</adminEmail>";
    print "<earliestDatestamp>1970-01-01T00:00:00Z</earliestDatestamp>";
    print "<deletedRecord>transient</deletedRecord>";
    print "<granularity>YYYY-MM-DDThh:mm:ssZ</granularity>";
    print "<compression>deflate</compression>";

    print "<description>";
    print "<oai-identifier xmlns=\"http://www.openarchives.org/OAI/2.0/oai-identifier\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.openarchives.org/OAI/2.0/oai-identifier http://www.openarchives.org/OAI/2.0/oai-identifier.xsd\">";
    print "<scheme>oai</scheme>";
    print "<repositoryIdentifier>" . $config->{"repositoryId"} . "</repositoryIdentifier>";
    print "<delimiter>:</delimiter>";
    print "<sampleIdentifier>oai:" . $config->{"repositoryId"} . ":1</sampleIdentifier>";
    print "</oai-identifier>";
    print "</description>";

    if ($config->{"friends"}) {
      print "<description>";
      print "<friends xmlns=\"http://www.openarchives.org/OAI/2.0/friends/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.openarchives.org/OAI/2.0/friends/ http://www.openarchives.org/OAI/2.0/friends.xsd\">";
      for($i = 0; $i < @{$config->{"friends"}}; $i++) {
        print "<baseURL>" . $config->{"friends"}->[$i] . "</baseURL>";
      }
      print "</friends>";
      print "</description>";
    }
    print "</Identify>";
    print "</OAI-PMH>";
  }
}


#Request functions - return xml formatted text
sub OAIgetRecord {
  $records = getRecordsNew($repository);
  $metadataPrefix = $oai{'metadataPrefix'};
  $identifier = $oai{'identifier'};
  @parts = (split ':', $identifier);

  $root_name = $metadataFormats->{$metadataPrefix}->{"rootElementName"}; 
  if (!$identifier || !$metadataPrefix) {
    OAIXMLError("badArgument");
  } elsif (($parts[1] ne $config->{"repositoryId"}) || ($parts[0] ne "oai") || (!$records->{$parts[2]})) {
    OAIXMLError("idDoesNotExist", $identifier);
  } elsif (!$records->{$parts[2]}->{$root_name}->{"xml"}) {
    OAIXMLError("cannotDisseminateFormat", $metadataPrefix);
  } else {
    putGeneralHeader($HTTP_GET_VARS);
    print "<GetRecord><record>";
    print "<header>";
    print XMLtag("identifier", undef, "oai:" . $config->{"repositoryId"} . ":" . $parts[2]);
    print XMLtag("datestamp", undef, $records->{$parts[2]}->{$root_name}->{"created"});
    print (length($records->{$parts[2]}->{$root_name}->{"sett"}) > 0) ? XMLtag("setSpec", $records->{$parts[2]}->{$root_name}->{"sett"}) : "";
    print "</header>";
    print XMLtag("metadata", undef, $records->{$parts[2]}->{$root_name}->{"xml"});
    print "</record></GetRecord>";
    print "</OAI-PMH>";
  }
}

sub OAIlistRecords {
  $records = getRecordsNew($repository);
  $from = $oai{'from'};
  $until = $oai{'until'};
  $metadataPrefix = $oai{'metadataPrefix'};
  $set = $oai{'set'};
  $resumptionToken = $oai{'resumptionToken'};

  if ($resumptionToken) {
    $resumptionInfo = unserialize(gzuncompress(base64_decode(rawurldecode($resumptionToken))));
    $showmin = $resumptionInfo->{"showmin"};
    $set = $resumptionInfo->{"set"};
    $from = $resumptionInfo->{"from"};
    $until = $resumptionInfo->{"until"};
    $metadataPrefix = $resumptionInfo->{"metadataPrefix"};
  } else {
    $showmin = 0;
  }

  if (!$metadataPrefix) {
    OAIXMLError("cannotDisseminateFormat", $metadataPrefix);
  } elsif (!$metadataFormats->{$metadataPrefix}) {
    OAIXMLError("cannotDisseminateFormat", $metadataPrefix);
  } else {
    my $root_name = $metadataFormats->{$metadataPrefix}->{"rootElementName"};
    my @jorecords;
    my $response = "";
    foreach $kulcs (keys %$records) {
      if ($records->{$kulcs}->{$root_name}) {
        if (!$set || ($set && ($records->{$kulcs}->{$root_name}->{"sett"} eq $set))) {
          if ($records->{$kulcs}->{$root_name}->{"harvestable"} ne "n") {
            $created = $records->{$kulcs}->{$root_name}->{"created"};
            $modified = $records->{$kulcs}->{$root_name}->{"modified"};
            $deleted = $records->{$kulcs}->{$root_name}->{"deleted"};
            if ((!$from || ($from le $modified) || ($from le $deleted)) && (!$until || ($modified le $until) || ($deleted le $until))) {
              push @jorecords, $records->{$kulcs}->{$root_name};
            }
          }
        }
      }
    }
    $completeListSize = $#jorecords + 1;
    if ($config->{"resumptionSize"} > 0) {
      $shownum = $config->{"resumptionSize"};
    } else {
      $shownum = $completeListSize;
      if ($resumptionToken) {
        OAIXMLError("badResumptionToken", "This archive does not support the resumption method.");
        return 0;
      }
    }
    if ($completeListSize > 0) {
      if (($showmin < 0) || $showmin >= $completeListSize) {
        OAIXMLError("badResumptionToken");
        return 0;
      }
      @jorecords = sort { $a->{"id"} cmp $b->{"id"} } @jorecords;
      if ($showmin + $shownum > $completeListSize) {
        $shownum = $completeListSize - $showmin;
      }
      $showmax = $showmin + $shownum;

      for($i = $showmin; $i < $showmin + $shownum; $i++) {
        my $value = $jorecords[$i];
        $created = $value->{"created"};
        $modified = $value->{"modified"};
        $deleted = $value->{"deleted"};
        $response .= "<record>";
        if ($created ne $deleted) {
          $response .= "<header status=\"deleted\">";
          $response .= XMLtag("identifier", {"status" => "deleted"}, "oai:" . $config->{"repositoryId"} . ":" . $value->{"id"});
          $response .= XMLtag("datestamp", undef, $value->{"deleted"});
          $response .= (length($value->{"sett"}) > 0) ? XMLtag("setSpec", undef, $value->{"sett"}) : "";
          $response .= "</header>";
        } else {
          $response .= "<header>";
          $response .= XMLtag("identifier", undef, "oai:" . $config->{"repositoryId"} . ":" . $value->{"id"});
          $response .= XMLtag("datestamp", undef, $value->{"modified"});
          $response .= (length($value->{"sett"}) > 0) ? XMLtag("setSpec", undef, $value->{"sett"}) : "";
          $response .= "</header>";
          $response .= "<metadata>";
          $response .= $value->{"xml"};
          $response .= "</metadata>";
        }
        $response .= "</record>";
      }
      if ($config->{"resumptionSize"} > 0) {
        $response .= "<resumptionToken completeListSize=\"" . $completeListSize . "\" cursor=\"" . $showmin . "\" ";
        if ($showmax != $completeListSize) {
          $response .= "expirationDate=\"" . ISO8601Date(time() + $config->{"resumptionExpirationHours"} * 3600) . "\">";
          $response .= base64_encode(gzcompress(serialize({"showmin" => $showmax, "set" => $set, "from" => $from, "until" => $until, "metadataPrefix" => $metadataPrefix})));
        } else {
          $response .= ">";
        }
        $response .= "</resumptionToken>";
      }
    }
    if (length($response) > 0) {
      putGeneralHeader();
      print "<ListRecords>" . $response . "</ListRecords></OAI-PMH>";
    } else {
      OAIXMLError("noRecordsMatch");
    }
  }
}

sub OAIlistIdentifiers {
  $records = getRecordsNew($repository);
  $from = $oai{'from'};
  $until = $oai{'until'};
  $metadataPrefix = $oai{'metadataPrefix'};
  $set = $oai{'set'};
  $resumptionToken = $oai{'resumptionToken'};

  if ($resumptionToken) {
    $resumptionInfo = unserialize(gzuncompress(base64_decode(rawurldecode($resumptionToken))));
    $showmin = $resumptionInfo->{"showmin"};
    $set = $resumptionInfo->{"set"};
    $from = $resumptionInfo->{"from"};
    $until = $resumptionInfo->{"until"};
    $metadataPrefix = $resumptionInfo->{"metadataPrefix"};
  } else {
    $showmin = 0;
  }

  if (!$metadataPrefix) {
    OAIXMLError("cannotDisseminateFormat", $metadataPrefix);
  } elsif (!$metadataFormats->{$metadataPrefix}) {
    OAIXMLError("cannotDisseminateFormat", $metadataPrefix);
  } else {
    my $root_name = $metadataFormats->{$metadataPrefix}->{"rootElementName"};
    my @jorecords;
    my $response = "";
    foreach $kulcs (keys %$records) {
      if ($records->{$kulcs}->{$root_name}) {
        if (!$set || ($set && ($records->{$kulcs}->{$root_name}->{"sett"} eq $set))) {
          if ($records->{$kulcs}->{$root_name}->{"harvestable"} ne "n") {
            $created = $records->{$kulcs}->{$root_name}->{"created"};
            $modified = $records->{$kulcs}->{$root_name}->{"modified"};
            $deleted = $records->{$kulcs}->{$root_name}->{"deleted"};
            if ((!$from || ($from le $modified) || ($from le $deleted)) && (!$until || ($modified le $until) || ($deleted le $until))) {
              push @jorecords, $records->{$kulcs}->{$root_name};
            }
          }
        }
      }
    }
    $completeListSize = $#jorecords + 1;
    if ($config->{"resumptionSize"} > 0) {
      $shownum = $config->{"resumptionSize"};
    } else {
      $shownum = $completeListSize;
      if ($resumptionToken) {
        OAIXMLError("badResumptionToken", "This archive does not support the resumption method.");
        return 0;
      }
    }
    if ($completeListSize > 0) {
      if (($showmin < 0) || $showmin >= $completeListSize) {
        OAIXMLError("badResumptionToken");
        return 0;
      }
      @jorecords = sort { $a->{"id"} cmp $b->{"id"} } @jorecords;
      if ($showmin + $shownum > $completeListSize) {
        $shownum = $completeListSize - $showmin;
      }
      $showmax = $showmin + $shownum;

      for($i = $showmin; $i < $showmin + $shownum; $i++) {
        my $value = $jorecords[$i];
        $created = $value->{"created"};
        $modified = $value->{"modified"};
        $deleted = $value->{"deleted"};
        $response .= "<header>";
        if ($created ne $deleted) {
          $response .= XMLtag("identifier", {"status" => "deleted"}, "oai:" . $config->{"repositoryId"} . ":" . $value->{"id"});
          $response .= XMLtag("datestamp", undef, $value->{"deleted"});
        } else {
          $response .= XMLtag("identifier", undef, "oai:" . $config->{"repositoryId"} . ":" . $value->{"id"});
          $response .= XMLtag("datestamp", undef, $value->{"modified"});
        }
        $response .= (length($value->{"sett"}) > 0) ? XMLtag("setSpec", undef, $value->{"sett"}) : "";
        $response .= "</header>";
      }
      if ($config->{"resumptionSize"} > 0) {
        $response .= "<resumptionToken completeListSize=\"" . $completeListSize . "\" cursor=\"" . $showmin . "\" ";
        if ($showmax != $completeListSize) {
          $response .= "expirationDate=\"" . ISO8601Date(time() + $config->{"resumptionExpirationHours"} * 3600) . "\">";
          $response .= base64_encode(gzcompress(serialize({"showmin" => $showmax, "set" => $set, "from" => $from, "until" => $until, "metadataPrefix" => $metadataPrefix})));
        } else {
          $response .= ">";
        }
        $response .= "</resumptionToken>";
      }
    }
    if (length($response) > 0) {
      putGeneralHeader();
      print "<ListIdentifiers>" . $response . "</ListIdentifiers></OAI-PMH>";
    } else {
      OAIXMLError("noRecordsMatch");
    }
  }
}

sub OAIlistMetadataFormats {
  $identifier = $oai{'identifier'};

  $records = getRecordsNew($repository);
  @parts = (split ':', $identifier);
  {
    if ($identifier) {
      $records = getRecordsNew($repository);
      if (($parts[1] ne $config->{"repositoryId"}) || ($parts[0] ne "oai") || (!$records->{$parts[2]})) {
        OAIXMLError("idDoesNotExist", $identifier);
      } else {
        putGeneralHeader();
        print "<ListMetadataFormats>";
        foreach $key (sort keys %{$records->{$parts[2]}}) {
          if ($records->{$parts[2]}->{$key}->{"created"} eq $records->{$parts[2]}->{$key}->{"deleted"}) {
            print "<metadataFormat>";
            print XMLtag("metadataPrefix" , undef, $revmetadataFormats->{$key}->{"format.name"});
            print XMLtag("schema", undef, $revmetadataFormats->{$key}->{"schema"});
            print XMLtag("metadataNamespace", undef, $revmetadataFormats->{$key}->{"namespace"});
            print "</metadataFormat>";
          }
        }
        print "</ListMetadataFormats>";
        print "</OAI-PMH>";
      }
    } else {
      putGeneralHeader($HTTP_GET_VARS);
      print "<ListMetadataFormats>";
      foreach $key (sort keys %$metadataFormats) {
        print "<metadataFormat>";
        print XMLtag("metadataPrefix", undef, $key);
        print XMLtag("schema", undef, $metadataFormats->{$key}->{"schema"});
        print XMLtag("metadataNamespace", undef, $metadataFormats->{$key}->{"namespace"});
        print "</metadataFormat>";
      }
      print "</ListMetadataFormats>";
      print "</OAI-PMH>";
    }
  }
}

sub OAIlistSets {
  $resumptionToken = $oai{'resumptionToken'};

  if ($resumptionToken) {
    $resumptionInfo = unserialize(gzuncompress(base64_decode(rawurldecode($resumptionToken))));
    $showmin = $resumptionInfo->{"showmin"};
  } else {
    $showmin = 0;
  }
  # TODO: set kezeles: meg kell irni
  #$header = getHeaderFromCSV($repositoryfile);
  #$setek = unserialize(base64_decode($header[2]));
  $setek = undef;

  #if ($setek && array_key_exists("", $setek)) {
  #  unset($setek[""]);
  #}
  $completeListSize = $#setek;
  if ($config->{"resumptionSize"} > 0) {
    $shownum = $config->{"resumptionSize"};
  } else {
    $shownum = $completeListSize;
    if ($resumptionToken) {
      OAIXMLError("badResumptionToken");
      return 0;
    }
  }

  if ($completeListSize > 0) {
    if (($showmin < 0) || $showmin >= $completeListSize) {
      OAIXMLError("badResumptionToken");
      return 0;
    }
    if ($showmin + $shownum > $completeListSize) {
      $shownum = $completeListSize - $showmin;
    }
    $showmax = $showmin + $shownum;

    putGeneralHeader($HTTP_GET_VARS);
    print "<ListSets>";
    foreach my $set (keys %$setek) {
      $tomb = $setek->{$set};
      if (length($set) > 0) {
        print "<set><setSpec>" . $set . "</setSpec>";
        if ($tomb[0]) { print "<setName>" . $tomb[0] . "</setName>"; }
        if ($tomb[1]) { print "<setDescription>" . $tomb[1] . "</setDescription>"; }
        print "</set>";
      }
    }
    print "</ListSets>";
    print "</OAI-PMH>";
  } else {
    #if (!is_array($setek) || ($completeListSize == 0)) {
    OAIXMLError("noSetHierarchy", "This repository does not support sets");
  }
}

return 1;
